#ifndef VIETER_TREE
#define VIETER_TREE

#include <stdint.h>
#include <stdlib.h>

typedef struct vieter_tree vieter_tree;

typedef enum vieter_tree_error {
  vieter_tree_ok = 0,
  vieter_tree_already_present = 1,
  vieter_tree_not_present = 2
} vieter_tree_error;

/*
 * Allocate and initialize a new red-black binary tree.
 */
vieter_tree *vieter_tree_init();

/*
 * Deallocate a red-black binary tree.
 */
void vieter_tree_free(vieter_tree *tree);

/*
 * Insert a new entry into the tree.
 */
vieter_tree_error vieter_tree_insert(vieter_tree *tree, uint64_t key,
                                     void *data);

/*
 * Look for an entry in the tree.
 */
vieter_tree_error vieter_tree_search(void **out, vieter_tree *tree,
                                     uint64_t key);

/*
 * Remove a given entry from the tree.
 */
vieter_tree_error vieter_tree_remove(void **out, vieter_tree *tree,
                                     uint64_t key);

/*
 * Return how many entries are currently in the tree.
 */
uint64_t vieter_tree_size(vieter_tree *tree);

#endif
